from logger import Logger
from drivers import select_driver


class EventServer(Logger):
    """
    The event server uses an event driver to receive events, which are
    delegated to a widget tree (and eventually to gesture trackers).
    """
    def __init__(self, root_widget=None):
        # Root widget to which events are delegated
        self.root_widget = root_widget

        # Driver implementation that will be serving events
        self.event_driver = select_driver(self)

    def get_root_widget(self):
        return self.root_widget

    def set_root_widget(self, widget):
        self.root_widget = widget

    def delegate_event(self, event):
        """
        Delegate an event that has been triggered by the event driver to the
        widget tree.
        """
        self.root_widget.handle_event(event)

    def start(self):
        """
        Start the event loop. A root widget is needed to be able to delegate
        events, so check if it exists first.
        """
        if not self.root_widget:
            raise ValueError('Cannot start event server without root widget.')

        self.event_driver.start()

    def stop(self):
        """
        Stop the event loop.
        """
        self.event_driver.stop()

    def run(self):
        """
        Execute a single loop iteration of the event driver.
        """
        self.event_driver.run()
